@@ -21,7 +21,20 @@ maximise.likelihood <- function(data=stop("Data must be specified"), model=stop(
 		data <- na.omit(data)
 	}
 	
-	guitest <- testJAGS(silent=TRUE)
+	s.info <- Sys.info()
+	p.info <- .Platform
+	
+	if(as.numeric(paste(R.version$major, R.version$minor, sep="")) < 26){
+    	stop("Please update your version of R to the latest available (> 2.6.0 required)")
+	}
+	
+	os <- p.info$OS.type
+	username <- as.character(s.info["user"])
+	rversion <- R.version$version
+	gui <- p.info$GUI
+	p.type <- p.info$pkgType
+	
+	guitest <- c("R.GUI"=gui, "R.package.type"=p.type, "R.version"=list(rversion))
 	
 	#if(guitest$os=='windows' | (guitest$R.GUI == "AQUA" & guitest$R.package.type == "mac.binary")) eol <- "" else eol <- "                  \n"
 	if(guitest$R.GUI == "AQUA" & guitest$R.package.type == "mac.binary"){
@@ -34,21 +47,24 @@ maximise.likelihood <- function(data=stop("Data must be specified"), model=stop(
 	
 	if(silent==FALSE){
 		cat("Maximising the likelihood for the '", model, "' model.  This may take some time\n")
+		
+		flush.console()
 	}
 	
 	if(model=="P"){
 		if(is.na(mean)) mean <- mean(data)
 		
 		if(silent==TRUE){
-			f <- function(mean) return(likelihood(model=model, data=data, mean=mean, silent=TRUE))
+			f1 <- function(mean) return(max(-Inf, likelihood(model=model, data=data, mean=mean, silent=TRUE), na.rm=TRUE))
 		}else{
 			cat("\n\tmean\n", eol, "\t", signif(mean+(mean/100000), 4), eol, sep="")
-			f <- function(mean){
+			f1 <- function(mean){
 				cat(clearline, "\t", signif(mean+(mean/100000), 4), eol, sep="")
-				return(likelihood(model=model, data=data, mean=mean, silent=TRUE))
+				flush.console()
+				return(max(-Inf, likelihood(model=model, data=data, mean=mean, silent=TRUE), na.rm=TRUE))
 			}
 		}
-		maxim <- optimise(f, mean, lower=0, upper=max(data)*10, maximum=TRUE)
+		maxim <- optimise(f1, mean, lower=0, upper=max(data)*10, maximum=TRUE)
 		results <- c(mean=maxim$maximum)
 	}
 
@@ -57,19 +73,20 @@ maximise.likelihood <- function(data=stop("Data must be specified"), model=stop(
 		if(is.na(zi)) zi <- sum(data==0)/length(data)*100
 		
 		if(silent==TRUE){
-			f <- function(pars=c(NA, NA)){
+			f2 <- function(pars=c(NA, NA)){
 				if(pars[2] < 0 | pars[2] > 100 | pars[1] < 0) return(-Inf)
-				return(likelihood(model=model, data=data, mean=pars[1], zi=pars[2], silent=TRUE))
+				return(max(-Inf, likelihood(model=model, data=data, mean=pars[1], zi=pars[2], silent=TRUE), na.rm=TRUE))
 			}
 		}else{
 			cat("\n\tmean\t\tzi\n", eol, "\t", signif(mean+(mean/100000), 4), "\t\t", signif(zi+(zi/100000), 4), eol, sep="")
-			f <- function(pars=c(NA, NA)){
+			f2 <- function(pars=c(NA, NA)){
 				if(pars[2] < 0 | pars[2] > 100 | pars[1] < 0) return(-Inf)
 				cat(clearline, "\t", signif(pars[1]+(pars[1]/100000), 4), "\t\t", signif(pars[2]+(pars[2]/100000), 4), eol, sep="")
-				return(likelihood(model=model, data=data, mean=pars[1], zi=pars[2], silent=TRUE))
+				flush.console()
+				return(max(-Inf, likelihood(model=model, data=data, mean=pars[1], zi=pars[2], silent=TRUE), na.rm=TRUE))
 			}
 		}
-		maxim <- optim(c(mean, zi), f, control=list(fnscale=-1))
+		maxim <- optim(c(mean, zi), f2, control=list(fnscale=-1))
 		results <- c(mean=maxim$par[1], zi=maxim$par[2])
 	}
 	
@@ -78,19 +95,20 @@ maximise.likelihood <- function(data=stop("Data must be specified"), model=stop(
 		if(is.na(shape)) shape <- mean(data) / scale
 		
 		if(silent==TRUE){
-			f <- function(pars=c(NA, NA)){
+			f3 <- function(pars=c(NA, NA)){
 				if(pars[2] <= 0 | pars[1] <= 0) return(-Inf)
-				return(likelihood(model=model, data=data, shape=pars[1], scale=pars[2], silent=TRUE))
+				return(max(-Inf, likelihood(model=model, data=data, shape=pars[1], scale=pars[2], silent=TRUE), na.rm=TRUE))
 			}
 		}else{
 			cat("\n\tshape\t\tscale\n", eol, "\t", signif(shape, 4), "\t\t", signif(scale, 4), eol, sep="")
-			f <- function(pars=c(NA, NA)){
+			f3 <- function(pars=c(NA, NA)){
 				if(pars[2] <= 0 | pars[1] <= 0) return(-Inf)
 				cat(clearline, "\t", signif(pars[1], 4), "\t\t", signif(pars[2], 4), eol, sep="")
-				return(likelihood(model=model, data=data, shape=pars[1], scale=pars[2], silent=TRUE))
+				flush.console()
+				return(max(-Inf, likelihood(model=model, data=data, shape=pars[1], scale=pars[2], silent=TRUE), na.rm=TRUE))
 			}
 		}
-		maxim <- optim(c(shape, scale), f, control=list(fnscale=-1))
+		maxim <- optim(c(shape, scale), f3, control=list(fnscale=-1))
 		results <- c(shape=maxim$par[1], scale=maxim$par[2])
 	}
 	
@@ -100,19 +118,20 @@ maximise.likelihood <- function(data=stop("Data must be specified"), model=stop(
 		if(is.na(zi)) zi <- sum(data==0)/length(data)*100
 		
 		if(silent==TRUE){
-			f <- function(pars=c(NA, NA, NA)){
+			f4 <- function(pars=c(NA, NA, NA)){
 				if(pars[2] <= 0 | pars[1] <= 0 | pars[3] < 0 | pars[3] > 100) return(-Inf)
-				return(likelihood(model=model, data=data, shape=pars[1], scale=pars[2], zi=pars[3], silent=TRUE))
+				return(max(-Inf, likelihood(model=model, data=data, shape=pars[1], scale=pars[2], zi=pars[3], silent=TRUE), na.rm=TRUE))
 			}
 		}else{
 			cat("\n\tshape\t\tscale\t\tzi\n", eol, "\t", signif(shape, 4), "\t\t", signif(scale, 4), "\t\t", signif(zi, 4), eol, sep="")
-			f <- function(pars=c(NA, NA, NA)){
+			f4 <- function(pars=c(NA, NA, NA)){
 				if(pars[2] <= 0 | pars[1] <= 0 | pars[3] < 0 | pars[3] > 100) return(-Inf)
 				cat(clearline, "\t", signif(pars[1], 4), "\t\t", signif(pars[2], 4), "\t\t", signif(pars[3], 4), eol, sep="")
-				return(likelihood(model=model, data=data, shape=pars[1], scale=pars[2], zi=pars[3], silent=TRUE))
+				flush.console()
+				return(max(-Inf, likelihood(model=model, data=data, shape=pars[1], scale=pars[2], zi=pars[3], silent=TRUE), na.rm=TRUE))
 			}
 		}
-		maxim <- optim(c(shape, scale, zi), f, control=list(fnscale=-1))
+		maxim <- optim(c(shape, scale, zi), f4, control=list(fnscale=-1))
 		results <- c(shape=maxim$par[1], scale=maxim$par[2], zi=maxim$par[3])
 	}
 	
@@ -121,19 +140,20 @@ maximise.likelihood <- function(data=stop("Data must be specified"), model=stop(
 		if(is.na(shape)) shape <- mean(data)
 		
 		if(silent==TRUE){
-			f <- function(pars=c(NA, NA)){
+			f5 <- function(pars=c(NA, NA)){
 				if(pars[2] <= 0 | pars[1] <= 0) return(-Inf)
-				return(likelihood(model=model, data=data, shape=pars[1], scale=pars[2], silent=TRUE))
+				return(max(-Inf, likelihood(model=model, data=data, shape=pars[1], scale=pars[2], silent=TRUE), na.rm=TRUE))
 			}
 		}else{
 			cat("\n\tshape\t\tscale\n", eol, "\t", signif(shape, 4), "\t\t", signif(scale, 4), eol, sep="")
-			f <- function(pars=c(NA, NA)){
+			f5 <- function(pars=c(NA, NA)){
 				if(pars[2] <= 0 | pars[1] <= 0) return(-Inf)
 				cat(clearline, "\t", signif(pars[1], 4), "\t\t", signif(pars[2], 4), eol, sep="")
-				return(likelihood(model=model, data=data, shape=pars[1], scale=pars[2], silent=TRUE))
+				flush.console()
+				return(max(-Inf, likelihood(model=model, data=data, shape=pars[1], scale=pars[2], silent=TRUE), na.rm=TRUE))
 			}
 		}
-		maxim <- optim(c(shape, scale), f, control=list(fnscale=-1))
+		maxim <- optim(c(shape, scale), f5, control=list(fnscale=-1))
 		results <- c(shape=maxim$par[1], scale=maxim$par[2])
 	}
 	
@@ -143,19 +163,20 @@ maximise.likelihood <- function(data=stop("Data must be specified"), model=stop(
 		if(is.na(zi)) zi <- sum(data==0)/length(data)*100
 		
 		if(silent==TRUE){
-			f <- function(pars=c(NA, NA, NA)){
+			f6 <- function(pars=c(NA, NA, NA)){
 				if(pars[2] <= 0 | pars[1] <= 0 | pars[3] < 0 | pars[3] > 100) return(-Inf)
-				return(likelihood(model=model, data=data, shape=pars[1], scale=pars[2], zi=pars[3], silent=TRUE))
+				return(max(-Inf, likelihood(model=model, data=data, shape=pars[1], scale=pars[2], zi=pars[3], silent=TRUE), na.rm=TRUE))
 			}
 		}else{
 			cat("\n\tshape\t\tscale\t\tzi\n", eol, "\t", signif(shape, 4), "\t\t", signif(scale, 4), "\t\t", signif(zi, 4), eol, sep="")
-			f <- function(pars=c(NA, NA, NA)){
+			f6 <- function(pars=c(NA, NA, NA)){
 				if(pars[2] <= 0 | pars[1] <= 0 | pars[3] < 0 | pars[3] > 100) return(-Inf)
 				cat(clearline, "\t", signif(pars[1], 4), "\t\t", signif(pars[2], 4), "\t\t", signif(pars[3], 4), eol, sep="")
-				return(likelihood(model=model, data=data, shape=pars[1], scale=pars[2], zi=pars[3], silent=TRUE))
+				flush.console()
+				return(max(-Inf, likelihood(model=model, data=data, shape=pars[1], scale=pars[2], zi=pars[3], silent=TRUE), na.rm=TRUE))
 			}
 		}
-		maxim <- optim(c(shape, scale, zi), f, control=list(fnscale=-1))
+		maxim <- optim(c(shape, scale, zi), f6, control=list(fnscale=-1))
 		results <- c(shape=maxim$par[1], scale=maxim$par[2], zi=maxim$par[3])
 	}
 	
@@ -164,19 +185,20 @@ maximise.likelihood <- function(data=stop("Data must be specified"), model=stop(
 		if(is.na(variance)) variance <- var(data)
 		
 		if(silent==TRUE){
-			f <- function(pars=c(NA, NA)){
+			f7 <- function(pars=c(NA, NA)){
 				if(pars[2] < 0 | pars[1] < 0) return(-Inf)
-				return(likelihood(model=model, data=data, mean=pars[1], variance=pars[2], silent=TRUE))
+				return(max(-Inf, likelihood(model=model, data=data, mean=pars[1], variance=pars[2], silent=TRUE), na.rm=TRUE))
 			}
 		}else{
 			cat("\n\tmean\t\tvariance\n", eol, "\t", signif(mean, 4), "\t\t", signif(variance, 4), eol, sep="")
-			f <- function(pars=c(NA, NA)){
+			f7 <- function(pars=c(NA, NA)){
 				if(pars[2] <= 0 | pars[1] < 0) return(-Inf)
 				cat(clearline, "\t", signif(pars[1], 4), "\t\t", signif(pars[2], 4), eol, sep="")
-				return(likelihood(model=model, data=data, mean=pars[1], variance=pars[2], silent=TRUE))
+				flush.console()
+				return(max(-Inf, likelihood(model=model, data=data, mean=pars[1], variance=pars[2], silent=TRUE), na.rm=TRUE))
 			}
 		}
-		maxim <- optim(c(mean, variance), f, control=list(fnscale=-1))
+		maxim <- optim(c(mean, variance), f7, control=list(fnscale=-1))
 		results <- c(mean=maxim$par[1], variance=maxim$par[2])
 	}
 	
@@ -186,19 +208,20 @@ maximise.likelihood <- function(data=stop("Data must be specified"), model=stop(
 		if(is.na(zi)) zi <- sum(data==0)/length(data)*100
 		
 		if(silent==TRUE){
-			f <- function(pars=c(NA, NA, NA)){
+			f8 <- function(pars=c(NA, NA, NA)){
 				if(pars[2] < 0 | pars[1] < 0 | pars[3] < 0 | pars[3] > 100) return(-Inf)
-				return(likelihood(model=model, data=data, mean=pars[1], variance=pars[2], zi=pars[3], silent=TRUE))
+				return(max(-Inf, likelihood(model=model, data=data, mean=pars[1], variance=pars[2], zi=pars[3], silent=TRUE), na.rm=TRUE))
 			}
 		}else{
 			cat("\n\tmean\t\tvariance\t\tzi\n", eol, "\t", signif(mean, 4), "\t\t", signif(variance, 4), "\t\t", signif(zi, 4), eol, sep="")
-			f <- function(pars=c(NA, NA, NA)){
+			f8 <- function(pars=c(NA, NA, NA)){
 				if(pars[2] < 0 | pars[1] < 0 | pars[3] < 0 | pars[3] > 100) return(-Inf)
 				cat(clearline, "\t", signif(pars[1], 4), "\t\t", signif(pars[2], 4), "\t\t", signif(pars[3], 4), eol, sep="")
-				return(likelihood(model=model, data=data, mean=pars[1], variance=pars[2], zi=pars[3], silent=TRUE))
+				flush.console()
+				return(max(-Inf, likelihood(model=model, data=data, mean=pars[1], variance=pars[2], zi=pars[3], silent=TRUE), na.rm=TRUE))
 			}
 		}
-		maxim <- optim(c(mean, variance, zi), f, control=list(fnscale=-1))
+		maxim <- optim(c(mean, variance, zi), f8, control=list(fnscale=-1))
 		results <- c(mean=maxim$par[1], variance=maxim$par[2], zi=maxim$par[3])
 	}
 	
